@@ -42,7 +42,6 @@
 #include <fcntl.h>
 #include <io.h>
 
-
 bool_t have_date = False;
 bool_t have_from = False;
 #ifdef HASTO_OPTION
@@ -53,6 +52,7 @@ bool_t minus_v = False;
 bool_t override_from = False;
 bool_t rewrite_domain = False;
 bool_t use_tls = False;			/* Use SSL to transfer mail to HUB */
+bool_t use_starttls = False;		/* SSL only after STARTTLS (RFC2487) */
 bool_t use_cert = False;		/* Use a certificate to transfer SSL mail */
 
 #define ARPADATE_LENGTH 32		/* Current date in RFC format */
@@ -110,6 +110,8 @@ void log_event(int priority, char *format, ...)
 	va_end(ap);
 
 #ifdef LOGFILE
+	FILE *fp;
+
 	if((fp = fopen("/tmp/ssmtp.log", "a")) != (FILE *)NULL) {
 		(void)fprintf(fp, "%s\n", buf);
 		(void)fclose(fp);
@@ -125,11 +127,16 @@ void log_event(int priority, char *format, ...)
 #else
 	openlog("sSMTP", LOG_PID, LOG_MAIL);
 #endif
-	syslog(priority, buf);
+	syslog(priority, "%s", buf);
 	closelog();
 #endif
 }
 
+void smtp_write(int fd, char *format, ...);
+int smtp_read(int fd, char *response);
+int smtp_read_all(int fd, char *response);
+int smtp_okay(int fd, char *response);
+
 /*
 dead_letter() -- Save stdin to ~/dead.letter if possible
 */
@@ -200,7 +207,7 @@ void die(char *format, ...)
 	va_end(ap);
 
 	(void)fprintf(stderr, "%s: %s\n", prog, buf);
-	log_event(LOG_ERR, buf);
+	log_event(LOG_ERR, "%s", buf);
 
 	/* Send message to dead.letter */
 	(void)dead_letter();
@@ -372,7 +379,7 @@ void revaliases(struct passwd *pw)
 	FILE *fp;
 
 	/* Try to open the reverse aliases file */
-	if((fp = fopen(REVALIASES_FILE, "r"))) {
+	if((fp = fopen(REVALIASES_FILE, "rt"))) {
 		/* Search if a reverse alias is defined for the sender */
 		while(fgets(buf, sizeof(buf), fp)) {
 			/* Make comments invisible */
@@ -786,7 +793,7 @@ bool_t read_config()
 	char buf[(BUF_SZ + 1)], *p, *q, *r;
 	FILE *fp;
 
-	if((fp = fopen(CONFIGURATION_FILE, "r")) == NULL) {
+	if((fp = fopen(CONFIGURATION_FILE, "rt")) == NULL) {
 		return(False);
 	}
 
@@ -887,6 +894,7 @@ bool_t read_config()
 				}
 				else {
 					use_tls = False;
+					use_starttls = False;
 				}
 
 				if(log_level > 0) { 
@@ -894,6 +902,20 @@ bool_t read_config()
 						"Set UseTLS=\"%s\"\n", use_tls ? "True" : "False");
 				}
 			}
+			else if(strcasecmp(p, "UseSTARTTLS") == 0) {
+				if(strcasecmp(q, "YES") == 0) {
+					use_starttls = True;
+					use_tls = True;
+				}
+				else {
+					use_starttls = False;
+				}
+
+				if(log_level > 0) { 
+					log_event(LOG_INFO,
+						"Set UseSTARTTLS=\"%s\"\n", use_tls ? "True" : "False");
+				}
+			}
 			else if(strcasecmp(p, "UseTLSCert") == 0) {
 				if(strcasecmp(q, "YES") == 0) {
 					use_cert = True;
@@ -945,6 +967,7 @@ int smtp_open(char *host, int port)
 
 #ifdef HAVE_SSL
 	int err;
+	char buf[(BUF_SZ + 1)];
 
 	/* Init SSL stuff */
 	SSL_CTX *ctx;
@@ -1044,6 +1067,33 @@ int smtp_open(char *host, int port)
 	if(use_tls == True) {
 		log_event(LOG_INFO, "Creating SSL connection to host");
 
+		if (use_starttls == True)
+		{
+			use_tls=False; /* need to write plain text for a while */
+
+			if (smtp_okay(s, buf))
+			{
+				smtp_write(s, "EHLO %s", hostname);
+				if (smtp_okay(s, buf)) {
+					smtp_write(s, "STARTTLS"); /* assume STARTTLS regardless */
+					if (!smtp_okay(s, buf)) {
+						log_event(LOG_ERR, "STARTTLS not working");
+						return(-1);
+					}
+				}
+				else
+				{
+					log_event(LOG_ERR, "Invalid response: %s (%s)", buf, hostname);
+				}
+			}
+			else
+			{
+				log_event(LOG_ERR, "Invalid response SMTP Server (STARTTLS)");
+				return(-1);
+			}
+			use_tls=True; /* now continue as normal for SSL */
+		}
+
 		ssl = SSL_new(ctx);
 		if(!ssl) {
 			log_event(LOG_ERR, "SSL not working");
@@ -1057,7 +1107,7 @@ int smtp_open(char *host, int port)
 			return(-1);
 		}
 
-		if(log_level > 0) { 
+		if(log_level > 0 || 1) {
 			log_event(LOG_INFO, "SSL connection using %s",
 				SSL_get_cipher(ssl));
 		}
@@ -1215,7 +1265,9 @@ int ssmtp(char *argv[])
 	get_arpadate(arpadate);
 
 	if(read_config() == False) {
-		log_event(LOG_INFO, "%s/ssmtp.conf not found", SSMTPCONFDIR);
+	/* Cygwin change: mention /usr/bin/ssmtp-config */
+		log_event(LOG_INFO, "%s/ssmtp.conf not found -- please run /usr/bin/ssmtp-config", SSMTPCONFDIR);
+		die("%s/ssmtp.conf not found -- please run /usr/bin/ssmtp-config", SSMTPCONFDIR);
 	}
 
 	if((p = strtok(pw->pw_gecos, ";,"))) {
@@ -1253,8 +1305,10 @@ int ssmtp(char *argv[])
 	if((sock = smtp_open(mailhost, port)) == -1) {
 		die("Cannot open %s:%d", mailhost, port);
 	}
-	else if(smtp_okay(sock, buf) == False) {
-		die("Invalid response SMTP server");
+	else if (use_starttls == False) /* no initial response after STARTTLS */
+	{
+		if(smtp_okay(sock, buf) == False)
+			die("Invalid response SMTP server");
 	}
 
 	/* If user supplied username and password, then try ELHO */
@@ -1273,7 +1327,7 @@ int ssmtp(char *argv[])
 	/* Try to log in if username was supplied */
 	if(auth_user) {
 #ifdef MD5AUTH
-		if(auth_pass == (char *)NULL)
+		if(auth_pass == (char *)NULL) {
 			auth_pass = strdup("");
 		}
 
@@ -1287,7 +1341,7 @@ int ssmtp(char *argv[])
 			strncpy(challenge, strchr(buf,' ') + 1, sizeof(challenge));
 
 			memset(buf, 0, sizeof(buf));
-			crammd5(challenge, authUsername, authPassword, buf);
+			crammd5(challenge, auth_user, auth_pass, buf);
 		}
 		else {
 #endif
